#
# Gowalla support for Gwibber
#
# Derived from Foursquare support (foursquare.py)
# 
# Copyright 2010, Tom "spot" Callaway <tcallawa@redhat.com>
#
# Permission to use, copy, modify, and distribute this software and its
# documentation for any purpose and without fee is hereby granted,
# provided that the above copyright notice appear in all copies and that
# both that copyright notice and this permission notice appear in
# supporting documentation.
# 
# THE AUTHOR PROVIDES THIS SOFTWARE ''AS IS'' AND ANY EXPRESSED OR
# IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
# OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
# IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

import network, json, util, htmllib, re
from util import log
from util import exceptions
from gettext import lgettext as _
from kitchen.text.converters import to_unicode
log.logger.name = "Gowalla"

PROTOCOL_INFO = {
  "name": "Gowalla",
  "version": "1.0",
  
  "config": [
    "private:password",
    "receive_enabled",
    "username",
    "color",
  ],

  "authtype": "none",
  "color": "#f7911f",

  "features": [
    "receive",
  ],

  "default_streams": [
    "receive",
  ],
}

URL_PREFIX = "https://api.gowalla.com"

class Client:
  def __init__(self, acct):
    self.account = acct

  def _message(self, data):

    m = {}; 
    m["mid"] = to_unicode(str(data["user"]["first_name"]) + str(data["user"]["last_name"]))
    m["service"] = u"gowalla"
    m["account"] = self.account["id"]
    m["time"] = util.parsetime(data["created_at"])

    messagetext = u""
    text = u""
    if data.has_key("spot"):
        if data.has_key("message"):
            messagetext += data["message"] + "<br/><br/>"
            text += data["message"] + "\n"
	m["url"] = to_unicode("<a href='http://gowalla.com%s'>" % data["spot"]["url"])
	m["url"] = to_unicode("http://gowalla.com%s" % data["spot"]["url"])
        img = to_unicode("<table><tr><td><img src='%s'/></td><td>" % data["spot"]["image_url"])
        messagetext += img + "Checked in at <a href='" + m["url"] + "'>" + to_unicode(data["spot"]["name"]) + "</a>"
	text += u"Checked in at " + to_unicode(data["spot"]["name"])
    else:
        if data.has_key("message"):
            messagetext += to_unicode(data["message"]) + u"<br/><br/>"
            text += to_unicode(data["message"]) + u"\n"
        else:
            text= u"Checked in off the grid"
            messagetext= u"<table><tr><td><img src='http://gowalla.com/favicon.ico'/></td><td>Checked in off the grid"

    m["text"] = text
    m["content"] = messagetext + u"</td></tr></table>"
    m["html"] = messagetext + u"</td></tr></table>"

    m["sender"] = {}
    m["sender"]["image"] = to_unicode(data["user"]["image_url"])
    m["sender"]["url"] = to_unicode("http://gowalla.com%s" % data["user"]["url"])
    fullname = u""
    if data["user"].has_key("first_name"):
        fullname += to_unicode(data["user"]["first_name"]) + u" "
    if data["user"].has_key("last_name"):
        fullname += to_unicode(data["user"]["last_name"])

    m["sender"]["name"] = fullname
    m["sender"]["nick"] = fullname
        
    m["source"] = u"<a href='http://gowalla.com/'>Gowalla</a>"

    return m

  def _get(self, path, parse="message", post=False, single=False, **args):
    url = "/".join((URL_PREFIX, path))

    data = network.Download(url, util.compact(args) or None, post,
        self.account["username"], self.account["password"]).get_json()

    checkins = data["activity"]
    if single: return [getattr(self, "_%s" % parse)(checkins)]
    if parse: return [getattr(self, "_%s" % parse)(m) for m in checkins]
    else: return []

  def __call__(self, opname, **args):
    return getattr(self, opname)(**args)

  def receive(self):
    return self._get("users/%s/activity/friends" % self.account["username"])

